# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import attribute as _attribute
from openstack.tests.functional import base


class TestAttribute(base.BaseFunctionalTest):
    """Test attribute CRUD.

    Attributes are key-value pairs attached to a deployable. The fake
    driver creates a deployable on startup which we use as the parent.
    """

    def setUp(self):
        super().setUp()
        self.require_service('accelerator')
        deployables = list(self.operator_cloud.accelerator.deployables())
        self.assertGreater(
            len(deployables),
            0,
            'no deployables found: is the fake driver enabled?',
        )
        self.deployable_id = deployables[0].id

    def _delete_attribute(self, attr):
        self.operator_cloud.accelerator.delete_attribute(attr)
        attrs = list(self.operator_cloud.accelerator.attributes())
        uuids = [a.uuid for a in attrs]
        self.assertNotIn(attr.uuid, uuids)

    def test_attribute(self):
        # TODO(melwitt): unskip once LP#2158996 is fixed
        self.skipTest(
            'Cyborg attribute API requires internal DB integer for '
            'deployable_id (LP#2158996)'
        )
        attr = self.operator_cloud.accelerator.create_attribute(
            deployable_id=self.deployable_id,
            key='test_key',
            value='test_value',
        )
        self.assertIsInstance(attr, _attribute.Attribute)
        self.addCleanup(self._delete_attribute, attr)

        # Get
        got = self.operator_cloud.accelerator.get_attribute(attr.uuid)
        self.assertIsInstance(got, _attribute.Attribute)
        self.assertEqual('test_key', got.key)
        self.assertEqual('test_value', got.value)

        # List
        attrs = list(self.operator_cloud.accelerator.attributes())
        uuids = [a.uuid for a in attrs]
        self.assertIn(attr.uuid, uuids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class BuildInfo(resource.Resource):
    base_path = '/build-info'
    resource_key = 'build_info'

    # Capabilities
    allow_fetch = True

    # Properties
    #: String representation of the API build version
    api = resource.Body('api')
    #: String representation of the engine build version
    engine = resource.Body('engine')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter

from openstack.baremetal.v1 import _common
from openstack import exceptions
from openstack import resource
from openstack import utils


class Runbook(_common.Resource):
    resources_key = 'runbooks'
    base_path = '/runbooks'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_patch = True
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'detail',
        fields={'type': _common.fields_type},
    )

    # Runbooks is available since 1.92
    # The description and traits fields added in 1.112
    _max_microversion = '1.112'
    name = resource.Body('name')
    #: Timestamp at which the runbook was created.
    created_at = resource.Body('created_at')
    #: A human readable description of the runbook. Introduced in API
    #: microversion 1.112.
    description = resource.Body('description')
    #: A set of one or more arbitrary metadata key and value pairs.
    extra = resource.Body('extra')
    #: A list of relative links. Includes the self and bookmark links.
    links = resource.Body('links', type=list)
    #: A set of physical information of the runbook.
    steps = resource.Body('steps', type=list)
    #: Indicates whether the runbook is publicly accessible.
    public = resource.Body('public', type=bool)
    #: The name or ID of the project that owns the runbook.
    owner = resource.Body('owner', type=str)
    #: The traits of the runbook. A node may only be acted upon by a runbook
    #: whose traits intersect with the traits of the node. Read-only here, use
    #: the trait methods to modify it. Introduced in API microversion 1.112.
    traits = resource.Body('traits', type=list)
    #: Timestamp at which the runbook was last updated.
    updated_at = resource.Body('updated_at')
    #: The UUID of the resource.
    id = resource.Body('uuid', alternate_id=True)

    def add_trait(self, session: adapter.Adapter, trait: str) -> None:
        """Add a trait to the runbook.

        :param session: The session to use for making this request.
        :param trait: The trait to add to the runbook.
        :returns: ``None``
        """
        session = self._get_session(session)
        version = utils.pick_microversion(
            session, _common.RUNBOOK_TRAITS_VERSION
        )
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits', trait)
        response = session.put(
            request.url,
            json=None,
            headers=request.headers,
            microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
        )

        msg = f"Failed to add trait {trait} for runbook {self.id}"
        exceptions.raise_from_response(response, error_message=msg)

        self.traits = list(set(self.traits or ()) | {trait})

    def remove_trait(self, session: adapter.Adapter, trait: str) -> None:
        """Remove a trait from the runbook.

        Removing a trait the runbook does not have is a no-op on the server
        side.

        :param session: The session to use for making this request.
        :param trait: The trait to remove from the runbook.
        :returns: ``None``
        """
        session = self._get_session(session)
        version = utils.pick_microversion(
            session, _common.RUNBOOK_TRAITS_VERSION
        )
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits', trait)

        response = session.delete(
            request.url,
            headers=request.headers,
            microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
        )

        msg = f"Failed to remove trait {trait} from runbook {self.id}"
        exceptions.raise_from_response(response, error_message=msg)

        if self.traits:
            self.traits = list(set(self.traits) - {trait})

    def set_traits(self, session: adapter.Adapter, traits: list[str]) -> None:
        """Set traits for the runbook.

        Removes any existing traits and adds the traits passed in to this
        method. Pass an empty list to remove all traits.

        :param session: The session to use for making this request.
        :param traits: list of traits to set on the runbook.
        :returns: ``None``
        """
        session = self._get_session(session)
        version = utils.pick_microversion(
            session, _common.RUNBOOK_TRAITS_VERSION
        )
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits')

        body = {'traits': traits}

        response = session.put(
            request.url,
            json=body,
            headers=request.headers,
            microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
        )

        msg = f"Failed to set traits for runbook {self.id}"
        exceptions.raise_from_response(response, error_message=msg)

        self.traits = traits



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import deployable as _deployable
from openstack.tests.functional import base


class TestDeployable(base.BaseFunctionalTest):
    """Test deployables discovered by the fake driver.

    The fake driver creates one device with one deployable on startup,
    so listing and getting deployables should always return results
    without any test setup.
    """

    def setUp(self):
        super().setUp()
        self.require_service('accelerator')

    def test_deployable(self):
        # List
        deployables = list(self.operator_cloud.accelerator.deployables())
        self.assertGreater(
            len(deployables),
            0,
            'no deployables found: is the fake driver enabled?',
        )
        self.assertIsInstance(deployables[0], _deployable.Deployable)

        # Get
        deployable = self.operator_cloud.accelerator.get_deployable(
            deployables[0].id
        )
        self.assertIsInstance(deployable, _deployable.Deployable)
        self.assertEqual(deployables[0].id, deployable.id)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import glob
import json
import os

import jsonschema
import referencing
from testtools import content

from openstack.config import defaults
from openstack.tests.unit.config import base

_CONFIG_DIR = os.path.dirname(os.path.realpath(defaults.__file__))


def _load_schema(filename):
    with open(os.path.join(_CONFIG_DIR, filename)) as f:
        return json.load(f)


class TestConfig(base.TestCase):
    def json_diagnostics(self, exc_info):
        self.addDetail('filename', content.text_content(self.filename))
        for error in sorted(
            self.validator.iter_errors(self.json_data), key=str
        ):
            self.addDetail('jsonschema', content.text_content(str(error)))

    def test_defaults_valid_json(self):
        schema = _load_schema('schema.json')
        self.validator = jsonschema.Draft202012Validator(schema)
        self.addOnException(self.json_diagnostics)

        self.filename = os.path.join(_CONFIG_DIR, 'defaults.json')
        with open(self.filename) as f:
            self.json_data = json.load(f)

        self.assertTrue(self.validator.is_valid(self.json_data))

    def test_clouds_valid_json(self):
        # the clouds.yaml schema reuses the single-cloud schema for each
        # 'clouds' entry via a cross-file $ref, so both schemas must be
        # registered for the reference to resolve
        schema = _load_schema('schema.json')
        clouds_schema = _load_schema('clouds-schema.json')
        registry = referencing.Registry().with_resources(
            [
                (schema['$id'], referencing.Resource.from_contents(schema)),
                (
                    clouds_schema['$id'],
                    referencing.Resource.from_contents(clouds_schema),
                ),
            ]
        )
        self.validator = jsonschema.Draft202012Validator(
            clouds_schema, registry=registry
        )
        self.addOnException(self.json_diagnostics)

        # USER_CONF is the comprehensive fixture used across the config
        # tests; it exercises the top-level sections (cache, client, metrics,
        # ansible), deprecated aliases and both region entry forms
        self.filename = 'clouds-schema.json'
        self.json_data = copy.deepcopy(base.USER_CONF)

        self.assertTrue(self.validator.is_valid(self.json_data))

    def test_vendors_valid_json(self):
        _schema_path = os.path.join(
            os.path.dirname(os.path.realpath(defaults.__file__)),
            'vendor-schema.json',
        )
        with open(_schema_path) as f:
            schema = json.load(f)
            self.validator = jsonschema.Draft202012Validator(schema)

        self.addOnException(self.json_diagnostics)

        _vendors_path = os.path.join(
            os.path.dirname(os.path.realpath(defaults.__file__)), 'vendors'
        )
        for self.filename in glob.glob(os.path.join(_vendors_path, '*.json')):
            with open(self.filename) as f:
                self.json_data = json.load(f)

                self.assertTrue(self.validator.is_valid(self.json_data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import device as _device
from openstack.tests.functional import base


class TestDevice(base.BaseFunctionalTest):
    """Test devices discovered by the fake driver.

    The fake driver creates one device on startup, so listing and
    getting devices should always return results without any test
    setup.
    """

    def setUp(self):
        super().setUp()
        self.require_service('accelerator')

    def test_device(self):
        # List
        devices = list(self.operator_cloud.accelerator.devices())
        self.assertGreater(
            len(devices),
            0,
            'no devices found: is the fake driver enabled?',
        )
        self.assertIsInstance(devices[0], _device.Device)

        # Get
        device = self.operator_cloud.accelerator.get_device(devices[0].uuid)
        self.assertIsInstance(device, _device.Device)
        self.assertEqual(devices[0].uuid, device.uuid)



# Copyright 2026 SoftBank corp.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, Self

from keystoneauth1 import adapter

from openstack import resource


class ResourceProviderTrait(resource.Resource):
    resource_key = None
    resources_key = None
    base_path = '/resource_providers/%(resource_provider_id)s/traits'

    _query_mapping = resource.QueryParameters(
        include_pagination_defaults=False
    )

    requires_id = False
    commit_method = 'PUT'

    # Capabilities

    allow_fetch = True
    allow_commit = True
    allow_delete = True

    # Properties
    #: The UUID of a resource provider.
    resource_provider_id = resource.URI('resource_provider_id')

    #: A consistent view marker that assists with the management of concurrent
    #: resource provider updates.
    resource_provider_generation = resource.Body(
        'resource_provider_generation',
        type=int,
    )

    #: Traits
    traits = resource.Body('traits', type=list, list_type=str)

    # Added in 1.6
    _max_microversion = '1.6'

    def commit(
        self,
        session: adapter.Adapter,
        prepend_key: bool = True,
        has_body: bool = True,
        retry_on_conflict: bool | None = None,
        base_path: str | None = None,
        *,
        microversion: str | None = None,
        **kwargs: Any,
    ) -> Self:
        # resource_provider_generation must always be provided on update, but
        # it will appear to be identical (by design) so we strip it. Prevent
        # this happening.
        self._body._dirty.add('resource_provider_generation')
        return super().commit(
            session,
            prepend_key=prepend_key,
            has_body=has_body,
            retry_on_conflict=retry_on_conflict,
            base_path=base_path,
            microversion=microversion,
            **kwargs,
        )

    def fetch(
        self,
        session: adapter.Adapter,
        requires_id: bool = False,
        base_path: str | None = None,
        error_message: str | None = None,
        skip_cache: bool = False,
        *,
        resource_response_key: str | None = None,
        microversion: str | None = None,
        **params: Any,
    ) -> Self:
        # The resource provider trait API doesn't require trait id information.
        return super().fetch(
            session,
            requires_id=False,
            base_path=base_path,
            error_message=error_message,
            skip_cache=skip_cache,
            resource_response_key=resource_response_key,
            microversion=microversion,
            **params,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.key_manager.v1 import _proxy
from openstack.key_manager.v1 import container
from openstack.key_manager.v1 import order
from openstack.key_manager.v1 import project_quota
from openstack.key_manager.v1 import quota
from openstack.key_manager.v1 import secret
from openstack.key_manager.v1 import secret_acl
from openstack.key_manager.v1 import secret_consumer
from openstack.key_manager.v1 import secret_store
from openstack.tests.unit import test_proxy_base


class TestKeyManagerProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestKeyManagerContainer(TestKeyManagerProxy):
    def test_server_create_attrs(self):
        self.verify_create(self.proxy.create_container, container.Container)

    def test_container_delete(self):
        self.verify_delete(
            self.proxy.delete_container, container.Container, False
        )

    def test_container_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_container, container.Container, True
        )

    def test_container_find(self):
        self.verify_find(self.proxy.find_container, container.Container)

    def test_container_get(self):
        self.verify_get(self.proxy.get_container, container.Container)

    def test_containers(self):
        self.verify_list(self.proxy.containers, container.Container)

    def test_container_update(self):
        self.verify_update(self.proxy.update_container, container.Container)


class TestKeyManagerOrder(TestKeyManagerProxy):
    def test_order_create_attrs(self):
        self.verify_create(self.proxy.create_order, order.Order)

    def test_order_delete(self):
        self.verify_delete(self.proxy.delete_order, order.Order, False)

    def test_order_delete_ignore(self):
        self.verify_delete(self.proxy.delete_order, order.Order, True)

    def test_order_find(self):
        self.verify_find(self.proxy.find_order, order.Order)

    def test_order_get(self):
        self.verify_get(self.proxy.get_order, order.Order)

    def test_orders(self):
        self.verify_list(self.proxy.orders, order.Order)

    def test_order_update(self):
        self.verify_update(self.proxy.update_order, order.Order)


class TestKeyManagerSecret(TestKeyManagerProxy):
    def test_secret_create_attrs(self):
        self.verify_create(self.proxy.create_secret, secret.Secret)

    def test_secret_delete(self):
        self.verify_delete(self.proxy.delete_secret, secret.Secret, False)

    def test_secret_delete_ignore(self):
        self.verify_delete(self.proxy.delete_secret, secret.Secret, True)

    def test_secret_find(self):
        self.verify_find(self.proxy.find_secret, secret.Secret)

    def test_secret_get(self):
        self.verify_get(self.proxy.get_secret, secret.Secret)
        self.verify_get_overrided(
            self.proxy, secret.Secret, 'openstack.key_manager.v1.secret.Secret'
        )

    def test_secrets(self):
        self.verify_list(self.proxy.secrets, secret.Secret)

    def test_secret_update(self):
        self.verify_update(self.proxy.update_secret, secret.Secret)


class TestKeyManagerQuota(TestKeyManagerProxy):
    def test_quota_get(self):
        self.verify_get(
            self.proxy.get_quota,
            quota.Quota,
            requires_id=False,
            method_args=[],
            expected_kwargs={'requires_id': False},
        )


class TestKeyManagerSecretStore(TestKeyManagerProxy):
    def test_secret_stores(self):
        self.verify_list(self.proxy.secret_stores, secret_store.SecretStore)


class TestKeyManagerProjectQuota(TestKeyManagerProxy):
    def test_project_quota_delete(self):
        self.verify_delete(
            self.proxy.delete_project_quota, project_quota.ProjectQuota, False
        )

    def test_project_quota_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_project_quota, project_quota.ProjectQuota, True
        )

    def test_project_quota_get(self):
        self.verify_get(
            self.proxy.get_project_quota, project_quota.ProjectQuota
        )

    def test_project_quota_update(self):
        self.verify_update(
            self.proxy.update_project_quota, project_quota.ProjectQuota
        )


class TestKeyManagerSecretACL(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_get_secret_acl(self):
        self.verify_get(
            self.proxy.get_secret_acl,
            secret_acl.SecretACL,
            expected_args=[None],
            expected_kwargs={
                "requires_id": False,
                "path_args": {"secret_id": "resource_id"},
            },
        )

    def test_set_secret_acl(self):
        self.verify_update(
            self.proxy.set_secret_acl,
            secret_acl.SecretACL,
            expected_args=[None],
            expected_kwargs={
                "requires_id": False,
                "path_args": {"secret_id": "resource_id"},
                "method": "PUT",
                "x": 1,
                "y": 2,
                "z": 3,
            },
        )

    def test_update_secret_acl(self):
        self.verify_update(
            self.proxy.update_secret_acl,
            secret_acl.SecretACL,
            expected_args=[None],
            expected_kwargs={
                "requires_id": False,
                "path_args": {"secret_id": "resource_id"},
                "method": "PATCH",
                "x": 1,
                "y": 2,
                "z": 3,
            },
        )

    def test_delete_secret_acl(self):
        self.verify_delete(
            self.proxy.delete_secret_acl,
            secret_acl.SecretACL,
            False,
            expected_args=[None],
            expected_kwargs={
                "requires_id": False,
                "path_args": {"secret_id": "resource_id"},
            },
        )


class TestKeyManagerSecretConsumer(TestKeyManagerProxy):
    def test_secret_consumer_create(self):
        self.verify_create(
            self.proxy.create_secret_consumer,
            secret_consumer.SecretConsumer,
            method_args=["secret-id"],
            method_kwargs={
                "service": "svc",
                "resource_type": "type",
                "resource_id": "rid",
            },
            expected_args=[],
            expected_kwargs={
                "secret_id": "secret-id",
                "service": "svc",
                "__conflicting_attrs": {"resource_type": "type"},
                "resource_id": "rid",
            },
        )

    def test_secret_consumers(self):
        self.verify_list(
            self.proxy.secret_consumers,
            secret_consumer.SecretConsumer,
            method_args=["secret-id"],
            expected_args=[],
            expected_kwargs={"secret_id": "secret-id"},
        )

    def test_secret_consumer_delete(self):
        self._verify(
            "openstack.key_manager.v1.secret_consumer.SecretConsumer.delete",
            self.proxy.delete_secret_consumer,
            method_args=["secret-id"],
            method_kwargs={
                "service": "svc",
                "resource_type": "type",
                "resource_id": "rid",
            },
            expected_args=[self.proxy],
        )


class TestProjectCleanup(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session, service_type='key-manager')

    def test_get_cleanup_dependencies(self):
        deps = self.proxy._get_cleanup_dependencies()
        self.assertIn('key_manager', deps)
        self.assertEqual([], deps['key_manager']['before'])

    @mock.patch.object(_proxy.Proxy, 'containers', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'delete_container', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'secrets', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'delete_secret', autospec=True)
    def test_service_cleanup(
        self,
        mock_del_secret,
        mock_secrets,
        mock_del_container,
        mock_containers,
    ):
        c = container.Container(id='c-1', name='test-container')
        s = secret.Secret(id='s-1', name='test-secret')
        mock_containers.return_value = [c]
        mock_secrets.return_value = [s]

        self.proxy._service_cleanup(
            dry_run=False,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=None,
        )

        mock_del_container.assert_called_once_with(self.proxy, c)
        mock_del_secret.assert_called_once_with(self.proxy, s)

    @mock.patch.object(_proxy.Proxy, 'containers', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'delete_container', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'secrets', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'delete_secret', autospec=True)
    def test_service_cleanup_dry_run(
        self,
        mock_del_secret,
        mock_secrets,
        mock_del_container,
        mock_containers,
    ):
        c = container.Container(id='c-1', name='test-container')
        s = secret.Secret(id='s-1', name='test-secret')
        mock_containers.return_value = [c]
        mock_secrets.return_value = [s]

        self.proxy._service_cleanup(
            dry_run=True,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=None,
        )

        mock_del_container.assert_not_called()
        mock_del_secret.assert_not_called()

    @mock.patch.object(_proxy.Proxy, 'containers', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'secrets', autospec=True)
    def test_service_cleanup_skip_secret(self, mock_secrets, mock_containers):
        mock_containers.return_value = []

        self.proxy._service_cleanup(
            dry_run=False,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=['key_manager.secret'],
        )

        mock_secrets.assert_not_called()
        mock_containers.assert_called_once_with(self.proxy)

    @mock.patch.object(_proxy.Proxy, 'containers', autospec=True)
    @mock.patch.object(_proxy.Proxy, 'secrets', autospec=True)
    def test_service_cleanup_skip_container(
        self, mock_secrets, mock_containers
    ):
        mock_secrets.return_value = []

        self.proxy._service_cleanup(
            dry_run=False,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=['key_manager.container'],
        )

        mock_containers.assert_not_called()
        mock_secrets.assert_called_once_with(self.proxy)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import endpoint_group as _endpoint_group
from openstack.tests.functional.identity.v3 import base


class TestEndpointGroup(base.BaseIdentityTest):
    def setUp(self):
        super().setUp()

        self.endpoint_group_name = self.getUniqueString('endpoint_group')
        self.endpoint_group_description = self.getUniqueString(
            'endpoint_group'
        )

        self.service_name = self.getUniqueString('service')
        self.service_type = self.getUniqueString('type')
        self.service = self.admin_identity_client.create_service(
            name=self.service_name,
            type=self.service_type,
        )
        self.addCleanup(
            self.admin_identity_client.delete_service, self.service
        )

        self.region_name = self.getUniqueString('region')
        self.region = self.admin_identity_client.create_region(
            name=self.region_name
        )
        self.addCleanup(self.admin_identity_client.delete_region, self.region)

        unique_base = self.getUniqueString('endpoint')
        self.test_url = f'https://{unique_base}.example.com/v1'
        self.updated_url = f'https://{unique_base}.example.com/v2'

        # Create public endpoint
        self.public_endpoint = self.admin_identity_client.create_endpoint(
            service_id=self.service.id,
            interface='public',
            url=self.test_url,
            region_id=self.region.id,
            is_enabled=True,
        )
        self.addCleanup(
            self.admin_identity_client.delete_endpoint, self.public_endpoint
        )

        # Create internal endpoint for filter testing
        self.internal_endpoint = self.admin_identity_client.create_endpoint(
            service_id=self.service.id,
            interface='internal',
            url=self.test_url,
            region_id=self.region.id,
        )
        self.addCleanup(
            self.admin_identity_client.delete_endpoint, self.internal_endpoint
        )

        self.initial_filters = {
            'interface': 'internal',
        }
        self.filters = {
            'interface': 'public',
            'service_id': self.service.id,
            'region_id': self.region.id,
        }

    def _delete_endpoint_group(self, endpoint_group):
        ret = self.admin_identity_client.delete_endpoint_group(endpoint_group)
        self.assertIsNone(ret)

    def test_endpoint_group(self):
        # Create endpoint group
        endpoint_group = self.admin_identity_client.create_endpoint_group(
            name=self.endpoint_group_name, filters=self.initial_filters
        )
        self.addCleanup(self._delete_endpoint_group, endpoint_group)
        self.assertIsInstance(endpoint_group, _endpoint_group.EndpointGroup)
        self.assertIsNotNone(endpoint_group.id)
        self.assertEqual(self.endpoint_group_name, endpoint_group.name)
        self.assertDictEqual(self.initial_filters, endpoint_group.filters)
        self.assertIsNone(endpoint_group.description)

        # Update endpoint group
        endpoint_group = self.admin_identity_client.update_endpoint_group(
            endpoint_group,
            description=self.endpoint_group_description,
            filters=self.filters,
        )
        self.assertIsInstance(endpoint_group, _endpoint_group.EndpointGroup)
        self.assertEqual(self.endpoint_group_name, endpoint_group.name)
        self.assertEqual(
            self.endpoint_group_description, endpoint_group.description
        )
        self.assertDictEqual(self.filters, endpoint_group.filters)

        # Get endpoint group by ID
        endpoint_group = self.admin_identity_client.get_endpoint_group(
            endpoint_group.id
        )
        self.assertIsInstance(endpoint_group, _endpoint_group.EndpointGroup)
        self.assertEqual(self.endpoint_group_name, endpoint_group.name)
        self.assertEqual(
            self.endpoint_group_description, endpoint_group.description
        )
        self.assertDictEqual(self.filters, endpoint_group.filters)

        # Find endpoint group
        found_endpoint_group = self.admin_identity_client.find_endpoint_group(
            endpoint_group.id, ignore_missing=False
        )
        self.assertIsInstance(
            found_endpoint_group, _endpoint_group.EndpointGroup
        )
        self.assertEqual(endpoint_group.id, found_endpoint_group.id)

        # Check filtered endpoints in endpoint group
        filtered_endpoints = list(
            self.admin_identity_client.endpoint_group_endpoints(endpoint_group)
        )
        endpoint_ids = {e.id for e in filtered_endpoints}
        self.assertIn(self.public_endpoint.id, endpoint_ids)
        self.assertNotIn(self.internal_endpoint.id, endpoint_ids)

        # List endpoint groups
        endpoint_groups = list(self.admin_identity_client.endpoint_groups())
        self.assertIsInstance(
            endpoint_groups[0], _endpoint_group.EndpointGroup
        )
        endpoint_group_ids = {eg.id for eg in endpoint_groups}
        self.assertIn(endpoint_group.id, endpoint_group_ids)

        # Test name filter
        endpoint_groups = list(
            self.admin_identity_client.endpoint_groups(
                name=endpoint_group.name
            )
        )
        self.assertIsInstance(
            endpoint_groups[0], _endpoint_group.EndpointGroup
        )
        endpoint_group_ids = {eg.id for eg in endpoint_groups}
        self.assertIn(endpoint_group.id, endpoint_group_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock
import uuid
import warnings

from openstack import exceptions
from openstack import resource
from openstack.tests.unit import base
from openstack.tests.unit import fakes


class Test_Exception(base.TestCase):
    def test_method_not_supported(self):
        class FakeResource(resource.Resource):
            pass

        exc = exceptions.MethodNotSupported(FakeResource, 'list')
        expected = (
            'The list method is not supported for '
            'openstack.tests.unit.test_exceptions.FakeResource'
        )
        self.assertEqual(expected, str(exc))


class Test_HttpException(base.TestCase):
    def setUp(self):
        super().setUp()
        self.message = 'mayday'
        self.response = fakes.FakeResponse(
            status_code=401,
            data={
                'error': {
                    'code': 401,
                    'message': (
                        'The request you have made requires authentication.'
                    ),
                    'title': 'Unauthorized',
                },
            },
        )

    def _do_raise(self, *args, **kwargs):
        raise exceptions.HttpException(*args, **kwargs)

    def test_message(self):
        exc = self.assertRaises(
            exceptions.HttpException,
            self._do_raise,
            self.message,
            response=self.response,
        )

        self.assertEqual(self.message, exc.message)

    def test_details(self):
        details = "some details"
        exc = self.assertRaises(
            exceptions.HttpException,
            self._do_raise,
            self.message,
            response=self.response,
            details=details,
        )

        self.assertEqual(self.message, exc.message)
        self.assertEqual(details, exc.details)

    def test_http_status(self):
        http_status = 123
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')

            exc = self.assertRaises(
                exceptions.HttpException,
                self._do_raise,
                self.message,
                http_status=http_status,
            )

        self.assertEqual(self.message, exc.message)
        self.assertEqual(http_status, exc.status_code)

        self.assertIn(
            "The 'http_status' parameter is unnecessary",
            str(w[-1]),
        )


class TestRaiseFromResponse(base.TestCase):
    def setUp(self):
        super().setUp()
        self.message = "Where is my kitty?"

    def _do_raise(self, *args, **kwargs):
        return exceptions.raise_from_response(*args, **kwargs)

    def test_raise_no_exception(self):
        response = mock.Mock()
        response.status_code = 200
        self.assertIsNone(self._do_raise(response))

    def test_raise_not_found_exception(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
            'x-openstack-request-id': uuid.uuid4().hex,
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(self.message, exc.message)
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(
            response.headers.get('x-openstack-request-id'), exc.request_id
        )

    def test_raise_bad_request_exception(self):
        response = mock.Mock()
        response.status_code = 400
        response.headers = {
            'content-type': 'application/json',
            'x-openstack-request-id': uuid.uuid4().hex,
        }
        exc = self.assertRaises(
            exceptions.BadRequestException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(self.message, exc.message)
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(
            response.headers.get('x-openstack-request-id'), exc.request_id
        )

    def test_raise_http_exception(self):
        response = mock.Mock()
        response.status_code = 403
        response.headers = {
            'content-type': 'application/json',
            'x-openstack-request-id': uuid.uuid4().hex,
        }
        exc = self.assertRaises(
            exceptions.HttpException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(self.message, exc.message)
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(
            response.headers.get('x-openstack-request-id'), exc.request_id
        )

    def test_raise_compute_format(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
        }
        response.json.return_value = {
            'itemNotFound': {
                'message': self.message,
                'code': 404,
            }
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(self.message, exc.details)
        self.assertIn(self.message, str(exc))

    def test_raise_network_format(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
        }
        response.json.return_value = {
            'NeutronError': {
                'message': self.message,
                'type': 'FooNotFound',
                'detail': '',
            }
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(self.message, exc.details)
        self.assertIn(self.message, str(exc))

    def test_raise_baremetal_old_format(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
        }
        response.json.return_value = {
            'error_message': json.dumps(
                {
                    'faultstring': self.message,
                    'faultcode': 'Client',
                    'debuginfo': None,
                }
            )
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(self.message, exc.details)
        self.assertIn(self.message, str(exc))

    def test_raise_baremetal_corrected_format(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
        }
        response.json.return_value = {
            'error_message': {
                'faultstring': self.message,
                'faultcode': 'Client',
                'debuginfo': None,
            }
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(self.message, exc.details)
        self.assertIn(self.message, str(exc))

    def test_raise_wsme_format(self):
        response = mock.Mock()
        response.status_code = 404
        response.headers = {
            'content-type': 'application/json',
        }
        response.json.return_value = {
            'faultstring': self.message,
            'faultcode': 'Client',
            'debuginfo': None,
        }
        exc = self.assertRaises(
            exceptions.NotFoundException,
            self._do_raise,
            response,
            error_message=self.message,
        )
        self.assertEqual(response.status_code, exc.status_code)
        self.assertEqual(self.message, exc.details)
        self.assertIn(self.message, str(exc))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any

from openstack.cloud import _utils
from openstack.cloud import openstackcloud
from openstack import exceptions
from openstack.orchestration.util import event_utils
from openstack.orchestration.v1 import stack as _stack


class OrchestrationCloudMixin(openstackcloud._OpenStackCloudMixin):
    def get_template_contents(
        self,
        template_file: str | None = None,
        template_url: str | None = None,
        template_object: str | None = None,
        files: dict[str, Any] | None = None,
    ) -> tuple[dict[str, Any], dict[str, Any] | None]:
        return self.orchestration.get_template_contents(
            template_file=template_file,
            template_url=template_url,
            template_object=template_object,
            files=files,
        )

    def create_stack(
        self,
        name: str,
        tags: list[str] | None = None,
        template_file: str | None = None,
        template_url: str | None = None,
        template_object: str | None = None,
        files: dict[str, Any] | None = None,
        rollback: bool = True,
        wait: bool = False,
        timeout: int | float = 3600,
        environment_files: list[str] | None = None,
        **parameters: Any,
    ) -> _stack.Stack | None:
        """Create a stack.

        :param name: Name of the stack.
        :param tags: List of tag(s) of the stack. (optional)
        :param template_file: Path to the template.
        :param template_url: URL of template.
        :param template_object: URL to retrieve template object.
        :param files: dict of additional file content to include.
        :param rollback: Enable rollback on create failure.
        :param wait: Whether to wait for the delete to finish.
        :param timeout: Stack create timeout in seconds.
        :param environment_files: Paths to environment files to apply.

        Other arguments will be passed as stack parameters which will take
        precedence over any parameters specified in the environments.

        Only one of template_file, template_url, template_object should be
        specified.

        :returns: a dict containing the stack description
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call
        """
        params: dict[str, Any] = {
            'tags': tags,
            'is_rollback_disabled': not rollback,
            'timeout_mins': timeout // 60,
            'parameters': parameters,
        }
        params.update(
            self.orchestration.read_env_and_templates(
                template_file=template_file,
                template_url=template_url,
                template_object=template_object,
                files=files,
                environment_files=environment_files,
            )
        )
        self.orchestration.create_stack(name=name, **params)
        if wait:
            event_utils.poll_for_events(self, stack_name=name, action='CREATE')
        return self.get_stack(name)

    def update_stack(
        self,
        name_or_id: str,
        template_file: str | None = None,
        template_url: str | None = None,
        template_object: str | None = None,
        files: dict[str, Any] | None = None,
        rollback: bool = True,
        tags: list[str] | None = None,
        wait: bool = False,
        timeout: int | float = 3600,
        environment_files: list[str] | None = None,
        **parameters: Any,
    ) -> _stack.Stack | None:
        """Update a stack.

        :param name_or_id: Name or ID of the stack to update.
        :param template_file: Path to the template.
        :param template_url: URL of template.
        :param template_object: URL to retrieve template object.
        :param files: dict of additional file content to include.
        :param rollback: Enable rollback on update failure.
        :param wait: Whether to wait for the delete to finish.
        :param timeout: Stack update timeout in seconds.
        :param environment_files: Paths to environment files to apply.

        Other arguments will be passed as stack parameters which will take
        precedence over any parameters specified in the environments.

        Only one of template_file, template_url, template_object should be
        specified.

        :returns: a dict containing the stack description
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API calls
        """
        params: dict[str, Any] = {
            'tags': tags,
            'is_rollback_disabled': not rollback,
            'timeout_mins': timeout // 60,
            'parameters': parameters,
        }
        params.update(
            self.orchestration.read_env_and_templates(
                template_file=template_file,
                template_url=template_url,
                template_object=template_object,
                files=files,
                environment_files=environment_files,
            )
        )
        if wait:
            # find the last event to use as the marker
            events = event_utils.get_events(
                self, name_or_id, event_args={'sort_dir': 'desc', 'limit': 1}
            )
            marker = events[0].id if events else None

        self.orchestration.update_stack(name_or_id, **params)

        if wait:
            event_utils.poll_for_events(
                self, name_or_id, action='UPDATE', marker=marker
            )
        return self.get_stack(name_or_id)

    def delete_stack(self, name_or_id: str, wait: bool = False) -> bool:
        """Delete a stack

        :param name_or_id: Stack name or ID.
        :param wait: Whether to wait for the delete to finish

        :returns: True if delete succeeded, False if the stack was not found.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call
        """
        stack = self.get_stack(name_or_id, resolve_outputs=False)
        if stack is None:
            self.log.debug("Stack %s not found for deleting", name_or_id)
            return False

        if wait:
            # find the last event to use as the marker
            events = event_utils.get_events(
                self, name_or_id, event_args={'sort_dir': 'desc', 'limit': 1}
            )
            marker = events[0].id if events else None

        self.orchestration.delete_stack(stack)

        if wait:
            try:
                event_utils.poll_for_events(
                    self, stack_name=name_or_id, action='DELETE', marker=marker
                )
            except exceptions.HttpException:
                pass
            stack = self.get_stack(name_or_id, resolve_outputs=False)
            if stack and stack['stack_status'] == 'DELETE_FAILED':
                raise exceptions.SDKException(
                    "Failed to delete stack {id}: {reason}".format(
                        id=name_or_id, reason=stack['stack_status_reason']
                    )
                )

        return True

    def search_stacks(
        self,
        name_or_id: str | None = None,
        filters: dict[str, Any] | str | None = None,
    ) -> list[_stack.Stack]:
        """Search stacks.

        :param name_or_id: Name or ID of the desired stack.
        :param filters: a dict containing additional filters to use. e.g.
                {'stack_status': 'CREATE_COMPLETE'}

        :returns: a list of ``openstack.orchestration.v1.stack.Stack``
            containing the stack description.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call.
        """
        stacks = self.list_stacks()
        return _utils._filter_list(stacks, name_or_id, filters)

    def list_stacks(self, **query: Any) -> list[_stack.Stack]:
        """List all stacks.

        :param query: Query parameters to limit stacks.

        :returns: a list of :class:`openstack.orchestration.v1.stack.Stack`
            objects containing the stack description.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call.
        """
        return list(self.orchestration.stacks(**query))

    def get_stack(
        self,
        name_or_id: str,
        filters: dict[str, Any] | str | None = None,
        resolve_outputs: bool = True,
    ) -> _stack.Stack | None:
        """Get exactly one stack.

        :param name_or_id: Name or ID of the desired stack.
        :param filters: a dict containing additional filters to use. e.g.
                {'stack_status': 'CREATE_COMPLETE'}
        :param resolve_outputs: If True, then outputs for this
                stack will be resolved

        :returns: a :class:`openstack.orchestration.v1.stack.Stack`
            containing the stack description
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call or if multiple matches are
            found.
        """

        # stack names are mandatory and enforced unique in the project
        # so a StackGet can always be used for name or ID.
        try:
            stack = self.orchestration.find_stack(
                name_or_id,
                ignore_missing=False,
                resolve_outputs=resolve_outputs,
            )
        except exceptions.NotFoundException:
            return None
        if stack.status == 'DELETE_COMPLETE':
            return None
        results = _utils._filter_list([stack], name_or_id, filters)
        return results[0] if results else None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import accelerator_request as _arq
from openstack.tests.functional import base


class TestAcceleratorRequest(base.BaseFunctionalTest):
    """Test accelerator requests (ARQs).

    ARQ creation requires a device profile to exist. The setUp creates
    one for use by the tests. ARQ binding is not tested here because it
    requires a Nova instance with an accelerator-aware flavor.
    """

    def setUp(self):
        super().setUp()
        self.require_service('accelerator')
        self.DP_NAME = self.getUniqueString()
        self.GROUPS = [
            {
                'resources:CUSTOM_ACCELERATOR_FPGA': '1',
                'trait:CUSTOM_FPGA_INTEL': 'required',
            }
        ]
        self.device_profile = (
            self.operator_cloud.accelerator.create_device_profile(
                name=self.DP_NAME,
                groups=self.GROUPS,
            )
        )
        self.addCleanup(
            self.operator_cloud.accelerator.delete_device_profile,
            self.device_profile,
        )

    def test_accelerator_request(self):
        # Create as regular user (project member)
        arq = self.user_cloud.accelerator.create_accelerator_request(
            device_profile_name=self.DP_NAME,
        )
        self.assertIsInstance(arq, _arq.AcceleratorRequest)
        self.assertIsNotNone(arq.uuid)

        # Get as regular user
        got = self.user_cloud.accelerator.get_accelerator_request(arq.uuid)
        self.assertIsInstance(got, _arq.AcceleratorRequest)
        self.assertEqual(arq.uuid, got.uuid)

        # List as regular user
        arqs = list(self.user_cloud.accelerator.accelerator_requests())
        uuids = [a.uuid for a in arqs]
        self.assertIn(arq.uuid, uuids)

        # Delete as regular user
        self.user_cloud.accelerator.delete_accelerator_request(arq)

        # Verify deletion
        arqs = list(self.user_cloud.accelerator.accelerator_requests())
        uuids = [a.uuid for a in arqs]
        self.assertNotIn(arq.uuid, uuids)

    def test_accelerator_request_admin(self):
        arq = self.operator_cloud.accelerator.create_accelerator_request(
            device_profile_name=self.DP_NAME,
        )
        self.assertIsInstance(arq, _arq.AcceleratorRequest)
        self.operator_cloud.accelerator.delete_accelerator_request(arq)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Mapping
from typing import Any
import warnings

from openstack.accelerator.v2 import accelerator_request as _arq
from openstack.accelerator.v2 import deployable as _deployable
from openstack.accelerator.v2 import device as _device
from openstack.accelerator.v2 import device_profile as _device_profile
from openstack.cloud import openstackcloud
from openstack import warnings as os_warnings


class AcceleratorCloudMixin(openstackcloud._OpenStackCloudMixin):
    def list_deployables(
        self, filters: dict[str, Any] | None = None
    ) -> list[_deployable.Deployable]:
        """List all available deployables.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``Deployable`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.deployables(**filters))

    def list_devices(
        self, filters: dict[str, Any] | None = None
    ) -> list[_device.Device]:
        """List all devices.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``Device`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.devices(**filters))

    def list_device_profiles(
        self, filters: dict[str, Any] | None = None
    ) -> list[_device_profile.DeviceProfile]:
        """List all device_profiles.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``DeviceProfile`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.device_profiles(**filters))

    def create_device_profile(
        self, attrs: Mapping[str, Any]
    ) -> _device_profile.DeviceProfile:
        """Create a device_profile.

        :param attrs: The info of device_profile to be created.
        :returns: An accelerator ``DeviceProfile`` objects.
        """
        return self.accelerator.create_device_profile(**attrs)

    def delete_device_profile(self, uuid: str, filters: Any = None) -> bool:
        """Delete a device_profile.

        :param uuid: The UUID of the device profile to be deleted.
        :param filters: dict of filter conditions to push down
        :returns: True if delete succeeded, False otherwise.
        """
        if filters is not None:
            warnings.warn(
                'The fields argument is a no-op and will be removed in a '
                'future release',
                os_warnings.RemovedInSDK50Warning,
            )

        self.accelerator.delete_device_profile(uuid)
        return True

    def list_accelerator_requests(
        self, filters: dict[str, Any] | None = None
    ) -> list[_arq.AcceleratorRequest]:
        """List all accelerator_requests.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``AcceleratorRequest`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.accelerator_requests(**filters))

    def delete_accelerator_request(
        self, uuid: str, filters: Any = None
    ) -> bool:
        """Delete a accelerator_request.

        :param uuid: The UUID of the accelerator request to be deleted.
        :param filters: dict of filter conditions to push down
        :returns: True if delete succeeded, False otherwise.
        """
        if filters is not None:
            warnings.warn(
                'The fields argument is a no-op and will be removed in a '
                'future release',
                os_warnings.RemovedInSDK50Warning,
            )

        self.accelerator.delete_accelerator_request(
            accelerator_request=uuid,
        )
        return True

    def create_accelerator_request(
        self, attrs: Mapping[str, Any]
    ) -> _arq.AcceleratorRequest:
        """Create an accelerator_request.

        :param attrs: The info of accelerator_request to be created.
        :returns: An accelerator ``AcceleratorRequest`` object.
        """
        return self.accelerator.create_accelerator_request(**attrs)

    def patch_accelerator_request(
        self, uuid: str, patch: list[dict[str, Any]]
    ) -> _arq.AcceleratorRequest:
        """Update an accelerator request.

        :param uuid: The UUID of the accelerator request to be updated.
        :param patch: The that will bind the accelerator.
        :returns: True if bind succeeded, False otherwise.
        """
        return self.accelerator.patch_accelerator_request(uuid, patch)

    def bind_accelerator_request(
        self, uuid: str, properties: list[dict[str, Any]]
    ) -> _arq.AcceleratorRequest:
        """Bind an accelerator to VM.

        A deprecated alias for `patch_accelerator_request`.
        """
        return self.patch_accelerator_request(uuid, properties)

    def unbind_accelerator_request(
        self, uuid: str, properties: list[dict[str, Any]]
    ) -> _arq.AcceleratorRequest:
        """Unbind an accelerator from VM.

        A deprecated alias for `patch_accelerator_request`.
        """
        return self.patch_accelerator_request(uuid, properties)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any

from openstack.network.v2 import sfc_service_graph
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE: dict[str, Any] = {
    "description": "",
    "project_id": "4ad57e7ce0b24fca8f12b9834d91079d",
    "tenant_id": "4ad57e7ce0b24fca8f12b9834d91079d",
    "port_chains": {
        "0e6b9678-19aa-11ee-97ae-a3cec2c2ac72": [
            "1e19c266-19aa-11ee-8e02-6fa0c9a9832d"
        ],
        "2a394dc8-19aa-11ee-b87e-7f24d71926f1": [
            "3299fcf6-19aa-11ee-9398-3f8c68c11209"
        ],
    },
    "id": "6ecd9cf3-ca64-46c7-863f-f2eb1b9e838a",
    "name": "service_graph",
}


class TestSfcServiceGraph(base.TestCase):
    def test_basic(self):
        sot = sfc_service_graph.SfcServiceGraph()
        self.assertEqual('service_graph', sot.resource_key)
        self.assertEqual('service_graphs', sot.resources_key)
        self.assertEqual('/sfc/service_graphs', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = sfc_service_graph.SfcServiceGraph(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['port_chains'], sot.port_chains)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                'description': 'description',
                'name': 'name',
                'project_id': 'project_id',
                'tenant_id': 'tenant_id',
            },
            sot._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.cloud import openstackcloud
from openstack.shared_file_system.v2 import (
    availability_zone as _availability_zone,
)


class SharedFileSystemCloudMixin(openstackcloud._OpenStackCloudMixin):
    def list_share_availability_zones(
        self,
    ) -> list[_availability_zone.AvailabilityZone]:
        """List all availability zones for the Shared File Systems service.

        :returns: A list of Shared File Systems Availability Zones.
        """
        return list(self.share.availability_zones())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import device_profile as _device_profile
from openstack.tests.functional import base


class TestDeviceProfile(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()
        self.require_service('accelerator')
        self.NAME = self.getUniqueString()
        self.GROUPS = [
            {
                'resources:CUSTOM_ACCELERATOR_FPGA': '1',
                'trait:CUSTOM_FPGA_INTEL': 'required',
            }
        ]
        self.device_profile = (
            self.operator_cloud.accelerator.create_device_profile(
                name=self.NAME,
                groups=self.GROUPS,
            )
        )
        self.addCleanup(self._delete_device_profile, self.device_profile)

    def _delete_device_profile(self, dp):
        self.operator_cloud.accelerator.delete_device_profile(dp)
        names = [
            d.name for d in self.operator_cloud.accelerator.device_profiles()
        ]
        self.assertNotIn(dp.name, names)

    def test_device_profile(self):
        # Get as regular user
        dp = self.user_cloud.accelerator.get_device_profile(
            self.device_profile.uuid
        )
        self.assertIsInstance(dp, _device_profile.DeviceProfile)
        self.assertEqual(self.NAME, dp.name)

        # List as regular user
        names = [
            dp.name for dp in self.user_cloud.accelerator.device_profiles()
        ]
        self.assertIn(self.NAME, names)

    def test_device_profile_admin(self):
        # Get
        dp = self.operator_cloud.accelerator.get_device_profile(
            self.device_profile.uuid
        )
        self.assertIsInstance(dp, _device_profile.DeviceProfile)
        self.assertEqual(self.NAME, dp.name)

        # List
        names = [
            dp.name for dp in self.operator_cloud.accelerator.device_profiles()
        ]
        self.assertIn(self.NAME, names)

        # Delete (create a separate one to avoid disrupting cleanup)
        name = self.getUniqueString()
        dp = self.operator_cloud.accelerator.create_device_profile(
            name=name,
            groups=self.GROUPS,
        )
        self._delete_device_profile(dp)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter

from openstack import exceptions
from openstack import resource
from openstack import utils


class EndpointGroup(resource.Resource):
    resource_key = 'endpoint_group'
    resources_key = 'endpoint_groups'
    base_path = '/OS-EP-FILTER/endpoint_groups'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'name', include_pagination_defaults=False
    )

    #: The description of the endpoint group. *Type: string*
    description = resource.Body('description')
    #: Describes the filtering performed by the endpoint group.
    #: The filter used must be an endpoint property, namely
    #: interface, service_id, or region.
    #: *Type: dict*
    filters = resource.Body('filters')
    #: The links for the endpoint group resource.
    links = resource.Body('links')
    #: The name of the endpoint group. *Type: string*
    name = resource.Body('name')

    def associate_project(
        self, session: adapter.Adapter, project_id: str
    ) -> None:
        """Associate project with endpoint group.

        :param session: The session to use for making this request.
        :param project_id: The ID of a project.
        :returns: None
        """
        url = utils.urljoin(
            '/OS-EP-FILTER/endpoint_groups',
            self.id,
            'projects',
            project_id,
        )
        response = session.put(url)
        exceptions.raise_from_response(response)

    def disassociate_project(
        self, session: adapter.Adapter, project_id: str
    ) -> None:
        """Disassociate project from endpoint group.

        :param session: The session to use for making this request.
        :param endpoint_id: The ID of a project.
        :returns: None
        """
        url = utils.urljoin(
            '/OS-EP-FILTER/endpoint_groups',
            self.id,
            'projects',
            project_id,
        )
        response = session.delete(url)
        exceptions.raise_from_response(response)


class ProjectEndpointGroup(EndpointGroup):
    base_path = '/OS-EP-FILTER/projects/%(project_id)s/endpoint_groups'

    #: The ID for the project from the URI of the resource
    project_id = resource.URI('project_id')

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = False
    allow_delete = False
    allow_list = True
